==> ecs-girvel-3.0.0.tar.gz: ecs-girvel-3.0.0/ecs/metasystem.py <==
from typing import TypeVar

from .entity import Entity

from .essentials import update, register, unregister
from .system import System


_TEntity = TypeVar("_TEntity", bound=Entity)


class MetasystemFacade(Entity):
    """Facade class containing all general ECS logic."""

    ecs_metasystem_facade_flag: None = None

    def __init__(self) -> None:
        """Initializes a new game; creates a metasystem."""

        @System
        def metasystem(system: System) -> None:
            update(system)

        self.__metasystem__ = metasystem

    def add(self, entity: _TEntity) -> _TEntity:
        """Registers entity as a member of ECS; sets entity's __metasystem__ attribute.

        Args:
            entity: entity to be added

        Returns:
            The same entity
        """

        if entity.__metasystem__ is not None:
            raise OwnershipException("Entity {entity} already belongs to a metasystem")

        entity.__metasystem__ = self.__metasystem__

        register(self.__metasystem__, entity)

        return entity

    def remove(self, entity: _TEntity) -> _TEntity:
        """Unregisters entity from the ECS.

        Args:
            entity: entity to be added

        Returns:
            The same entity
        """

        if entity.__metasystem__ is None:
            raise OwnershipException("Entity should belong to the metasystem to be deleted from it")

        unregister(self.__metasystem__, entity)
        return entity

    def update(self) -> None:
        """Updates all the systems once."""
        update(self.__metasystem__)

    def register_itself(self):
        """Allows system to access MetasystemFacade as entity. Call after adding systems."""
        register(self.__metasystem__, self)


class OwnershipException(Exception):
    pass


def exists(entity: "Entity") -> bool:
    """Determines whether the entity belongs to any ECS."""
    return entity.__metasystem__ is not None

==> ecs-girvel-3.0.1.tar.gz: ecs-girvel-3.0.1/ecs/metasystem.py <==
from typing import TypeVar

from .entity import Entity

from .essentials import update, register, unregister
from .system import System


_TEntity = TypeVar("_TEntity", bound=Entity)


class MetasystemFacade(Entity):
    """Facade class containing all general ECS logic."""

    __metasystem__: System
    ecs_metasystem_facade_flag: None = None

    def __init__(self) -> None:
        """Initializes a new game; creates a metasystem."""

        @System
        def metasystem(system: System) -> None:
            update(system)

        self.__metasystem__ = metasystem

    def add(self, entity: _TEntity) -> _TEntity:
        """Registers entity as a member of ECS; sets entity's __metasystem__ attribute.

        Args:
            entity: entity to be added

        Returns:
            The same entity
        """

        if entity.__metasystem__ is not None:
            raise OwnershipException(f"Can not add entity {entity} - it already belongs to a metasystem")

        entity.__metasystem__ = self.__metasystem__

        register(self.__metasystem__, entity)

        return entity

    def remove(self, entity: _TEntity) -> _TEntity:
        """Unregisters entity from the ECS.

        Args:
            entity: entity to be added

        Returns:
            The same entity
        """

        if entity.__metasystem__ is not self.__metasystem__:
            raise OwnershipException(
                f"Can not remove entity {entity} - it does not belong to metasystem {self.__metasystem__}"
            )

        unregister(self.__metasystem__, entity)
        return entity

    def update(self) -> None:
        """Updates all the systems once."""
        update(self.__metasystem__)

    def register_itself(self) -> None:
        """Allows system to access MetasystemFacade as entity. Call after adding systems."""
        register(self.__metasystem__, self)


class OwnershipException(Exception):
    pass


def exists(entity: "Entity") -> bool:
    """Determines whether the entity belongs to any ECS."""
    return entity.__metasystem__ is not None
